# energy units
GeV = 1.0
MeV = 1e-3*GeV
TeV = 1e3*GeV
PeV = 1e6*GeV

# length units
m   = 1.0
mm   = 1e-3*m
cm  = 1e-2*m
nm  = 1e-9*m
km  = 1e3*m
# time units
ns  = 1
us  = 1e3*ns
s   = 1e9*ns

# velocity
light_velocity_vacuum = 299792458*m/s

# phys constants
alpha_em = 0.0072973525693
